@@ -4,7 +4,7 @@ import copy
 from enum import IntEnum
 from typing import List, Optional, Tuple, Union
 
-from pylogos.query_graph.koutrika_query_graph import (Node, Predicate)
+from pylogos.query_graph.koutrika_query_graph import Node, Predicate
 
 
 class OperationType(IntEnum):
@@ -27,11 +27,14 @@ class SStrChar:
     :param str: _description_
     :type str: _type_
     """
-    def __init__(self, text: str, table_node: str, column_node: Optional[str]=None, op_type: Optional[OperationType]=None):
+
+    def __init__(
+        self, text: str, table_node: str, column_node: Optional[str] = None, op_type: Optional[OperationType] = None
+    ):
         self.ori_label = text
         self.text: str = text
-        self.table_node: str=table_node
-        self.column_node: Optional[str]=column_node
+        self.table_node: str = table_node
+        self.column_node: Optional[str] = column_node
         self.op_type: Optional[OperationType] = op_type
 
     def __str__(self):
@@ -45,7 +48,11 @@ class SStrChar:
 
     def __eq__(self, other):
         if type(other) == SStrChar:
-            return self.text == other.text and self.table_node == other.table_node and self.column_node == other.column_node
+            return (
+                self.text == other.text
+                and self.table_node == other.table_node
+                and self.column_node == other.column_node
+            )
         return False
 
     def __bool__(self):
@@ -56,15 +63,19 @@ class SStrChar:
         return str(self) in ["", " "]
 
     def is_same_root(self, other):
-        return self.ori_label == other.ori_label and \
-            remove_number_in_string(self.table_node) == remove_number_in_string(other.table_node) and \
-            remove_number_in_string(self.column_node) == remove_number_in_string(other.column_node)
+        return (
+            self.ori_label == other.ori_label
+            and remove_number_in_string(self.table_node) == remove_number_in_string(other.table_node)
+            and remove_number_in_string(self.column_node) == remove_number_in_string(other.column_node)
+        )
 
     def add_prefix(self, prefix: str) -> None:
         self.text = prefix + self.text
+        return self
 
     def add_suffix(self, suffix: str) -> None:
         self.text += suffix
+        return self
 
     def startswith(self, other):
         return self.text.startswith(other)
@@ -82,13 +93,13 @@ class SStrWord:
 
     def __hash__(self):
         return hash(tuple(self.items))
-    
+
     def __len__(self):
         return len(self.items)
 
     def __getitem__(self, idx):
         return self.items[idx]
-    
+
     def __eq__(self, other):
         if type(other) == SStrWord:
             return all([sstr1 == sstr2 for sstr1, sstr2 in zip(self.items, other.items)])
@@ -96,9 +107,9 @@ class SStrWord:
 
     def __add__(self, other: SStrWord):
         if type(other) == SStrWord:
-            return SStrWord(list(filter(lambda k: str(k) != '', self.items + other.items)))
+            return SStrWord(list(filter(lambda k: str(k) != "", self.items + other.items)))
         elif type(other) == SStrChar:
-            return SStrWord(list(filter(lambda k: str(k) != '', self.items + [other])))
+            return SStrWord(list(filter(lambda k: str(k) != "", self.items + [other])))
         raise TypeError(f"Cannot add {type(other)} to SStrs")
 
     def __bool__(self):
@@ -114,48 +125,50 @@ class SStrWord:
 
     def startswith(self, substring: str):
         return str(self).startswith(substring)
-    
+
     def endswith(self, other):
         return self.items[-1].endswith(other)
-    
+
     def is_same_root(cls, other):
         return all(sstr1.is_same_root(sstr2) for sstr1, sstr2 in zip(cls.items, other.items))
 
     def add_prefix(self, prefix: str) -> None:
         self.items[0].add_prefix(prefix)
+        return self
 
     def add_suffix(self, suffix: str) -> None:
         self.items[-1].add_suffix(suffix)
+        return self
 
 
 class SStrSen:
     def __init__(self, items: List[SStrWord]):
         self.items: List[SStrWord] = copy.deepcopy(items)
-    
+
     def __str__(self):
         return " ".join([str(sstr) for sstr in self.items])
-    
+
     def __hash__(self):
         return hash(tuple(self.items))
-    
+
     def __len__(self):
         return len(self.items)
-    
+
     def __getitem__(self, idx):
         return self.items[idx]
-    
+
     def __eq__(self, other):
         if type(other) == SStrSen:
             return all([sstr1 == sstr2 for sstr1, sstr2 in zip(self.items, other.items)])
         return False
-    
+
     def __add__(self, other: SStrSen):
         if type(other) == SStrSen:
-            return SStrSen(list(filter(lambda k: str(k) != '', self.items + other.items)))
+            return SStrSen(list(filter(lambda k: str(k) != "", self.items + other.items)))
         elif type(other) == SStrWord:
-            return SStrSen(list(filter(lambda k: str(k) != '', self.items + [other])))
+            return SStrSen(list(filter(lambda k: str(k) != "", self.items + [other])))
         elif type(other) == SStrChar:
-            return SStrSen(list(filter(lambda k: str(k) != '', self.items + [SStrWord([other])])))
+            return SStrSen(list(filter(lambda k: str(k) != "", self.items + [SStrWord([other])])))
         raise TypeError(f"Cannot add {type(other)} to SStrs")
 
     def __bool__(self):
@@ -189,7 +202,7 @@ class SStrSen:
             item += self.items[i]
         self.items = [item]
         return self
-    
+
     def get_sentence_mapping(self):
         """Output string and json object mapping the phrase information to the sentence
         :return: _description_
@@ -199,18 +212,21 @@ class SStrSen:
         cnt = 0
         for i, sstr_word in enumerate(self.items):
             for j, sstr_char in enumerate(sstr_word.items):
-                mapping.append({
-                    "start":cnt,
-                    "end": cnt+len(sstr_char),
-                    "type": int(sstr_char.op_type) if sstr_char.op_type else None,
-                    "table": sstr_char.table_node,
-                    "column": sstr_char.column_node if sstr_char.column_node else None,
-                })
+                mapping.append(
+                    {
+                        "start": cnt,
+                        "end": cnt + len(sstr_char),
+                        "type": int(sstr_char.op_type) if sstr_char.op_type else None,
+                        "table": sstr_char.table_node,
+                        "column": sstr_char.column_node if sstr_char.column_node else None,
+                    }
+                )
                 cnt += len(sstr_char)
                 if j < len(sstr_word.items):
                     cnt += 1
         return mapping
 
+
 class StringBuilder:
     def __init__(self, is_nested=False):
         self.is_nested: bool = is_nested
@@ -230,8 +246,7 @@ class StringBuilder:
             elif dont_use_of:
                 return desc + rel_sen
             else:
-                desc.add_suffix(" of")
-                return desc + rel_sen
+                return desc + rel_sen.add_prefix("of ")
         else:
             return desc
 
@@ -240,18 +255,18 @@ class StringBuilder:
         for i, t in enumerate(texts):
             if not t.is_empty_string and not new_str_sen.is_empty_string:
                 new_str_sen[-1].add_suffix(sep_str)
-            new_str_sen += t 
+            new_str_sen += t
         return new_str_sen
 
     def concate_by_comma_and(self, texts: SStrSen) -> SStrSen:
         for i in range(len(texts)):
-            if i < len(texts) -1 : # not the last one
-                if i < len(texts) - 2: # for every non-last two items
+            if i < len(texts) - 1:  # not the last one
+                if i < len(texts) - 2:  # for every non-last two items
                     texts[i].add_suffix(",")
-                elif i == len(texts) - 2: # for every second to last item
-                    if len(texts) == 2: # if there are only two items
+                elif i == len(texts) - 2:  # for every second to last item
+                    if len(texts) == 2:  # if there are only two items
                         texts[i].add_suffix(" and")
-                    else: # if there are more than two items
+                    else:  # if there are more than two items
                         texts[i][-1].add_suffix(", and")
                 else:
                     raise RuntimeError("Should not be here")
@@ -293,7 +308,7 @@ class StringBuilder:
                 if ttmp:
                     # Consider case where the label of the attribute is empty string
                     att_desc = self.description_of_rel(target_relation, ttmp, not att_flag)
-                    
+
                     text = self.concate_by([text, att_desc], ",")
                 self.projection = tmp
 
@@ -317,10 +332,9 @@ class StringBuilder:
                         atts_str = self.concate_by_comma_and(SStrSen([SStrWord([item]) for item in att_list]))
                         # Append to the string
                         texts += self.description_of_rel(key_rel, atts_str).combine_words()
-                    text.add_suffix(" for each")
-                    text += self.concate_by_comma_and(texts)
+                    text += self.concate_by_comma_and(texts).add_prefix("for each ")
                 self.grouping = tmp
-            
+
             if self.having:
                 tmp = []
                 texts = SStrSen.create_empty_object()
@@ -339,8 +353,10 @@ class StringBuilder:
                     text.add_suffix(",")
                     text += self.concate_by_comma_and(texts).add_prefix(f"considering only those groups whose ")
                 self.having = tmp
-            
-            assert grouping_flag or grouping_flag == having_flag, f"HAVING must be used with GROUP BY, but {grouping_flag} != {having_flag}"
+
+            assert (
+                grouping_flag or grouping_flag == having_flag
+            ), f"HAVING must be used with GROUP BY, but {grouping_flag} != {having_flag}"
 
             # Get join_conditions for target relation
             if self.join_conditions:
@@ -357,13 +373,21 @@ class StringBuilder:
                             texts += SStrSen([SStrWord([edge, rel2])])
                             # texts += self.concate_by([edge, rel2], " ")
                             # Search for correlation condition in the selection
-                            correlations_for_rel2 = [sel for sel in self.selection if sel[0] != sel[1] and sel[0] == target_relation and sel[1].is_same_root(rel2) and rel2 == sel[5]]
-                            assert len(correlations_for_rel2) <= 1, f"the number of correlation conditions for {rel2} is {len(correlations_for_rel2)} > 1"
+                            correlations_for_rel2 = [
+                                sel
+                                for sel in self.selection
+                                if sel[0] != sel[1]
+                                and sel[0] == target_relation
+                                and sel[1].is_same_root(rel2)
+                                and rel2 == sel[5]
+                            ]
+                            assert (
+                                len(correlations_for_rel2) <= 1
+                            ), f"the number of correlation conditions for {rel2} is {len(correlations_for_rel2)} > 1"
                             if correlations_for_rel2:
                                 selected_correlation = correlations_for_rel2[0]
                                 self.selection.remove(selected_correlation)
-                                texts.add_suffix(f" of the same")
-                                texts += selected_correlation[2]
+                                texts += selected_correlation[2].add_prefix("of the same ")
                                 # texts[-1] += f" of the same {selected_correlation[2]}"
 
                     # Append to the string of rel1 if it has incoming edge
@@ -378,8 +402,7 @@ class StringBuilder:
                     join_flag = True
                     if having_flag:
                         text.add_suffix(",")
-                    text.add_suffix(f" in which")
-                    text += target_relation
+                    text += target_relation.add_prefix("in which ")
                     texts.combine_words()
                 text += texts
                 self.join_conditions = tmp
@@ -401,8 +424,7 @@ class StringBuilder:
                         else:
                             val_desc = SStrSen([SStrWord([val])])
                         if val_parent:
-                            val_desc.add_suffix(f" of those")
-                            val_desc += val_parent
+                            val_desc += val_parent.add_prefix("of those ")
                         if att.is_empty_string and op.startswith("there"):
                             subwords += op
                             subwords += val_desc
@@ -448,10 +470,20 @@ class StringBuilder:
         # Create SStr objects
         rel_sstr = SStrChar(relation.label, relation.name, op_type=OperationType.Projection)
         att_sstr = SStrChar(attribute.label, relation.name, attribute.name, op_type=OperationType.Projection)
-        agg_sstr = SStrChar(agg_func, relation.name, attribute.name, op_type=OperationType.Projection) if agg_func else None
+        agg_sstr = (
+            SStrChar(agg_func, relation.name, attribute.name, op_type=OperationType.Projection) if agg_func else None
+        )
         self.projection.append((rel_sstr, att_sstr, agg_sstr))
 
-    def add_selection(self, reference_point: Node, relation: Node, attribute: Node, op: Predicate, operand: str, operand_parent: Optional[Node] = None) -> None:
+    def add_selection(
+        self,
+        reference_point: Node,
+        relation: Node,
+        attribute: Node,
+        op: Predicate,
+        operand: str,
+        operand_parent: Optional[Node] = None,
+    ) -> None:
         # Create SStr objects
         ref_sstr = SStrChar(reference_point.label, reference_point.name, op_type=OperationType.Selection)
         rel_sstr = SStrChar(relation.label, relation.name, op_type=OperationType.Selection)
@@ -470,10 +502,16 @@ class StringBuilder:
                 arg1 = operand.label
                 arg3 = operand.name
             operand_sstr = SStrChar(arg1, arg2, arg3, op_type=OperationType.Selection)
-            operand_parent_sstr = SStrChar(operand_parent.label, operand_parent.name, op_type=OperationType.Selection) if operand_parent else None
+            operand_parent_sstr = (
+                SStrChar(operand_parent.label, operand_parent.name, op_type=OperationType.Selection)
+                if operand_parent
+                else None
+            )
         self.selection.append((ref_sstr, rel_sstr, att_sstr, op_sstr, operand_sstr, operand_parent_sstr))
 
-    def add_join_conditions(self, reference_point: Node, relation1: Node, edge: str, relation2: Node, has_incoming_edge: bool) -> None:
+    def add_join_conditions(
+        self, reference_point: Node, relation1: Node, edge: str, relation2: Node, has_incoming_edge: bool
+    ) -> None:
         # Create SStr objects
         ref_sstr = SStrChar(reference_point.label, reference_point.name, op_type=OperationType.Selection)
         rel1_sstr = SStrChar(relation1.label, relation1.name, op_type=OperationType.Selection)
@@ -487,8 +525,10 @@ class StringBuilder:
         rel_sstr = SStrChar(relation.label, relation.name, op_type=OperationType.Grouping)
         att_sstr = SStrChar(attribute.label, relation.name, attribute.name, op_type=OperationType.Grouping)
         self.grouping.append((ref_sstr, rel_sstr, att_sstr))
-    
-    def add_having(self, reference_point: Node, relation: Node, attribute: Node, function: Node, edge: Predicate, value: Node) -> None:
+
+    def add_having(
+        self, reference_point: Node, relation: Node, attribute: Node, function: Node, edge: Predicate, value: Node
+    ) -> None:
         # Create SStr objects
         ref_sstr = SStrChar(reference_point.label, reference_point.name, op_type=OperationType.Selection)
         rel_sstr = SStrChar(relation.label, relation.name, op_type=OperationType.Selection)
@@ -496,9 +536,9 @@ class StringBuilder:
         func_sstr = SStrChar(function.label, relation.name, attribute.name, op_type=OperationType.Selection)
         edge_sstr = SStrChar(edge.label, relation.name, attribute.name, op_type=OperationType.Selection)
         val_sstr = SStrChar(value.label, relation.name, attribute.name, op_type=OperationType.Selection)
-        
+
         self.having.append((ref_sstr, rel_sstr, att_sstr, func_sstr, edge_sstr, val_sstr))
-    
+
     def add_ordering(self, phrase: str) -> None:
         pass
 
@@ -515,4 +555,4 @@ class StringBuilder:
         self.join_conditions.extend(string_builder.join_conditions)
         self.grouping.extend(string_builder.grouping)
         self.having.extend(string_builder.having)
-        return self
\ No newline at end of file
+        return self
